import * as ts from 'typescript';
import { dirname, resolve } from 'path-browserify';
import merge from 'deepmerge';

const deepMerge = (a, b) => merge(a, b, {
    arrayMerge: (dest, src) => [...dest, ...src],
});
/**
 * Reads any typescript configuration files for a given file, including the extends references
 * @param filePath the full file path to the file
 * @param defaultConfig optional default configuration
 * @param keepJson set to tru to avoid converting options
 * @returns the typescript configuration for the file, or undefined if this is not a typescript file
 */
const getTypescriptConfig = (filePath, defaultConfig, keepJson) => {
    var _a;
    if (typeof window !== 'undefined') {
        return defaultConfig;
    }
    const readConfigFile = (configPath) => {
        let config = ts.readConfigFile(configPath, ts.sys.readFile).config;
        if (config.extends) {
            const extendsPath = ts.findConfigFile(resolve(dirname(configPath), config.extends), ts.sys.fileExists);
            if (extendsPath && extendsPath !== configPath) {
                config = deepMerge(readConfigFile(extendsPath), config);
            }
        }
        return config;
    };
    const ext = ((_a = filePath.split('.').pop()) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || 'js';
    if (['ts', 'tsx'].indexOf(ext) !== -1) {
        let config = defaultConfig || {};
        const configPath = ts.findConfigFile(dirname(filePath), ts.sys.fileExists);
        if (configPath) {
            const fileConfig = readConfigFile(configPath);
            if (fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions) {
                config = deepMerge(config, fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions);
            }
            if (config.baseUrl) {
                config.baseUrl = resolve(dirname(configPath), config.baseUrl);
            }
        }
        if (keepJson) {
            return config;
        }
        const { options } = ts.convertCompilerOptionsFromJson(config, '.');
        return options;
    }
    return undefined;
};

export { getTypescriptConfig };
